class AdvancementPlan < ActiveRecord::Base

  has_many :portfolio_item_advancement_plans,
           class_name: 'Portfolio::ItemAdvancementPlan',
           foreign_key: :advancement_plan_id,
           inverse_of: :advancement_plan,
           dependent: :restrict_with_error

  has_many :portfolio_items, class_name: '::Portfolio::Item', through: :portfolio_item_advancement_plans

  has_many :advancement_plan_imgs, autosave: true, dependent: :destroy
  has_many :advancement_plan_institution_stocks, inverse_of: :advancement_plan, dependent: :destroy

  has_many :advancement_plan_borrows, dependent: :destroy

  belongs_to :institution

  accepts_nested_attributes_for :advancement_plan_institution_stocks

  mount_uploader :cover_image, LibraryImgUploader
  after_save do
    cover_image.versions.each do |name,version|
      unless version.file.nil?
        advancement_plan_img_attributes = {name: name,width_px: 0, height_px: 0, file_type: version.file.extension.downcase,source_url: version.url, is_local: true}
        existing=advancement_plan_imgs.where(name: name).take
        if existing.present?
          existing.update_columns(advancement_plan_img_attributes)
        else
          advancement_plan_imgs.create!(advancement_plan_img_attributes)
        end
      end
    end
  end

  validates_format_of :year, with: /\d{4}/
  validates_presence_of :title, :age_from, :age_to
  validates_inclusion_of :min_level, :max_level, in: [0,1,2,3]
  validates_numericality_of :age_from, :age_to
  validates_length_of :description, minimum: 50
  validates_length_of :abbreviation, maximum: 16
  validate { errors.add(:min_level, 'Förderstufenbereich nicht richtig angegeben') unless min_level.to_i < max_level.to_i }
  validate { errors.add(:age_from, 'Altersbereich nicht richtig angegeben') unless age_from.to_i < age_to.to_i }

  after_initialize  unless: :persisted? do
    self.min_level ||= 0
    self.max_level ||= 3
    self.age_from ||= 1
    self.age_to ||= 99
    self.no_image_url ||= '/files/no-advancement-plan-image.jpeg'
  end

  before_validation do
    if self.catalog_id.blank? && self.title.present?
      require 'digest/md5'
      string = Digest::MD5.hexdigest(self.title.gsub(/\s+/,'').downcase)
      map = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split(//);
      part = string[0..12].to_i(16)
      hash = ''
      while(part != 0)
        hash = String(map[part % map.length ] ) + hash;
        part = part / map.length;
      end
      self.catalog_id = "vsb-#{hash}";
    end
  end


  def short_title
    return abbreviation if title.length > 60 && abbreviation.present?
    return title[0..60] + " ..." if title.length > 60 && abbreviation.blank?
    return (abbreviation + ' ' + title) if title.length <= 60 && abbreviation.present?
    return title if title.length <= 60 && abbreviation.blank?
  end


  def thumbnail
    advancement_plan_imgs.to_a.keep_if{|i| i.name=='thumb' }.first
  end

  def can_be_deleted?
    ! portfolio_item_advancement_plans.any?
  end

  ######################################################################################################################
  ### Suggestions
  ######################################################################################################################
  # Schüler sollen Vorschläge für Literatur bekommen.
  # TODO: this is apparently a shared interface between diagnostic, advancement_plan and online_diagnostic.


  # get the matching clients for a specific advancement plan
  def suggested_clients()
    suggestions = Client
    # has stock in client institution
    suggestions = suggestions.where(institution_id: advancement_plan_institution_stocks.is_available.pluck(:institution_id))
    # client is in age range
    if age_to
      suggestions = suggestions.where('bday >= ?', (Date.today - age_to.to_i))
    end
    if age_from
      suggestions = suggestions.where('bday <= ?', (Date.today - age_from.to_i ))
    end
    # client is in level range
    suggestions = suggestions.where('portfolio_level >= ? AND portfolio_level <= ?', min_level ,max_level)
    suggestions.order(updated_at: :desc) # this is to have the most recently active Clients as a suggestions
  end

end
