class OnlineDiagnostic < ActiveRecord::Base

  has_many :portfolio_item_diagnostics, class_name: '::Portfolio::ItemClassDiagnostic',
           foreign_key: :diagnostic_id,
           inverse_of: :diagnostic,
           dependent: :restrict_with_error

  has_many :online_diagnostic_urls, class_name: '::OnlineDiagnosticUrl',
           foreign_key: :online_diagnostic_id,
           inverse_of: :online_diagnostic,
           dependent: :destroy
  accepts_nested_attributes_for :online_diagnostic_urls, reject_if: :all_blank

  has_many :portfolio_items, class_name: '::Portfolio::Item',
           through: :portfolio_item_diagnostics



  validates_presence_of :title, :age_from, :age_to
  validates_inclusion_of :min_level, :max_level, in: [0,1,2,3]
  validates_numericality_of :age_from, :age_to
  validates_length_of :abbreviation, maximum: 16
  validate { errors.add(:min_level, 'Förderstufenbereich nicht richtig angegeben') unless min_level.to_i <= max_level.to_i }
  validate { errors.add(:age_from, 'Altersbereich nicht richtig angegeben') unless age_from.to_i <= age_to.to_i }

  after_initialize  unless: :persisted? do
    self.min_level ||= 0
    self.max_level ||= 3
    self.age_from ||= 1
    self.age_to ||= 99
  end



  def short_title
    return abbreviation if title.length > 60 && abbreviation.present?
    return title[0..60] + " ..." if title.length > 60 && abbreviation.blank?
    return (abbreviation + ' ' + title) if title.length <= 60 && abbreviation.present?
    return title if title.length <= 60 && abbreviation.blank?
  end



  def can_be_deleted?
    ! portfolio_item_diagnostics.any?
  end

  ######################################################################################################################
  ### Suggestions
  ######################################################################################################################
  # Schüler sollen Vorschläge für Literatur bekommen.
  # TODO: this is apparently a shared interface between diagnostic, advancement_plan and online_diagnostic.

  # get the matching clients for a specific advancement plan
  def suggested_clients
    suggestions = Client
    # client is in age range
    # client is in age range
    if age_to
      suggestions = suggestions.where('bday >= ?', (Date.today - age_to.to_i))
    end
    if age_from
      suggestions = suggestions.where('bday <= ?', (Date.today - age_from.to_i ))
    end
    # client is in level range
    suggestions = suggestions.where('portfolio_level >= ? AND portfolio_level <= ?', min_level ,max_level)
    suggestions.order(updated_at: :desc) # this is to have the most recently active Clients as a suggestions
  end


end
