module SystemTasks
  class Base < ActiveRecord::Base
    self.table_name = 'system_tasks'

    belongs_to :target, polymorphic: true
    belongs_to :client # can be null
    belongs_to :institution # can be null

    def escalations
      @escalations ||= _escalations
      @escalations
    end

    before_save do
      if escalation_level.blank?
        self.escalation_level=0
      end
    end

    after_save do
      # escalations are done here, directly after save and in cron.rake on an daily basis
      self.class.escalate
    end

    def _escalations
      []
    end

    def prio
      [[(escalation_level-1),0].max,2].min
    end

    def escalate
      if escalations.size <= escalation_level
        Rails.logger.debug "ESCALATION: #{self.class.name}<##{id}> stop on level #{escalation_level} of #{escalations.size} levels"
        self.next_escalation = nil
        save
      else
        notified = {}
        escalations.each_with_index do |escalation,level|
          if level < escalation_level
            escalation.each do |escalator,recipients|
              recipients.reject!(&:nil?)
              notified[escalator] ||= []
              r_uni = recipients.uniq{|r| "#{r.class.name}.#{r.id}" }
              notified[escalator] += r_uni
              Rails.logger.debug  "ESCALATION: #{self.class.name}<##{id}> skipping #{r_uni.size} already informed recipients on level #{level}"
            end
          end
          if level == escalation_level
            escalation.each do |escalator,recipients|
              notified[escalator] ||= []
              recipients.reject!(&:nil?)
              (recipients.uniq{|r| "#{r.class.name}.#{r.id}" } - notified[escalator] ).each do |recipient|
                next unless recipient
                escalator.new(recipient,self).execute
                Rails.logger.debug "ESCALATION: #{self.class.name}<##{id}> Executing #{escalator.inspect} for recipient #{recipient.inspect}"
              end
            end
            self.next_escalation = next_escalation + escalation_delay.seconds
            self.escalation_level += 1
            Rails.logger.debug  "ESCALATION: #{self.class.name}<##{id}> Setting next excalation to #{next_escalation} and current escalation level to #{escalation_level}"
            save
            break
          end
        end
      end
    end

    def is_manual_confirmable?
      raise 'Implementation missing'
    end

    class << self
      # this can run from a cronjob. The force flag is only for testing.
      def escalate( force=false )
        available.each do |klass|
          query = klass.all
          query = query.where('next_escalation < ?',Time.current) unless force
          query.each do |task|
            task.escalate
          end
        end
      end


      def available
        [
            SystemTasks::BorrowAdvancementPlan,SystemTasks::BorrowDiagnostic,SystemTasks::ClassDiagnosticResultIncomplete,
            SystemTasks::ClientClientGroupMissing,SystemTasks::ClientConfidentialityDocumentMissing,
            SystemTasks::DiagnosticMeasurementResultIncomplete,SystemTasks::DiagnosticResultIncomplete,
            SystemTasks::EducationalCourseViolations,SystemTasks::ManagementConference,SystemTasks::ParentTeacherConference,
            SystemTasks::PortfolioRuleNotifications::Document,SystemTasks::PortfolioRuleNotifications::Level,
            SystemTasks::PortfolioRuleNotifications::RequiredStep,
            SystemTasks::PortfolioRuleNotifications::RuleDescription,SystemTasks::PortfolioRuleNotifications::Steps
        ]
      end

      def subscribe
        available.each do |klass|
          klass.subscribe_subclass
        end
      end

      # Example:
      # [{
      #   klass: "portfolio/items/*"
      #   callback: "after_destroy"
      #   handler: 'handler_method_1'
      # }]
      #
      # Means: any class in portfolio/items will cause handler_method_1 to be executed on after destroy
      #
      # Example:
      # [{
      #   klass: "portfolio/items/client"
      #   callback: "after_save"
      #   handler: 'handler_method_2'
      # }]
      #
      # Means: ::Portfolio::Items::Client will trigger handler_method_2 to be executed on after save
      def subscribe_to
        []
      end


      def subscribe_subclass
        beeing_executed ={}
        subscribe_to.each do |options|
          if options[:klass].include?('*') # we do not support full regex, only * for a glob like behavior
            event = Regexp.new("#{options[:klass].gsub(/\*/, '.*?')}#{Regexp.escape(".#{options[:callback]}.active_record")}")
          else
            event = "#{options[:klass]}.#{options[:callback]}.active_record"
          end
          handler_method = options[:handler]
          ::ActiveSupport::Notifications.subscribe(event) do |name, start, finish, id, payload|
            unless beeing_executed[handler_method]
              beeing_executed[handler_method] = true
              begin
                send handler_method, payload[:model]
              rescue ArgumentError => e
                Rails.logger.error "#{name}.notification_handler #{handler_method} #{e}"
              end
              beeing_executed[handler_method] = false
            end
          end
        end
      end


    end

  end
end